/**
 * Module that knows how to locate initiatives to be removed
 */
const { searchItems } = require("@esri/arcgis-rest-items");

const initiativeQuery = 'typekeywords:hubInitiative';
const initiativeHelper = require('./initiative');
const async = require('async');
const concurrency = 4;
module.exports = {
  process: function (orgId, requestOptions) {
    const owner = requestOptions.authentication.username;
    let orgInitiativeQuery =`${initiativeQuery} AND orgid: ${orgId} AND owner: ${owner}`;
    console.info(`Searching for "${orgInitiativeQuery}"`);
    // Do a search for initiatives w/o the doNotDelete typekeyword
    // We only need to get ONE at this point, b/c we're just looking
    // for the total number of items, so if it's > 100, we can spread this over
    // a number of requests and do that work in a queue
    return searchItems({
      searchForm: {
        q: orgInitiativeQuery,
        start: 1,
        num: 1
      },
      ...requestOptions
    })
      .then((result) => {
        // now we have the total number of items matching that query, so
        // we can create an array of promises for the paged requests
        let promises = [];
        let step = 100;
        let start = 1;
        for (start; start < result.total; start = start + step){
          promises.push(searchItems({searchForm: { q: orgInitiativeQuery, start:start, num:100}, ...requestOptions}));
        }
        // this will either resolve, with an array of results
        // or reject if any of the promises reject
        return Promise.all(promises);
      })
      .then((searchPromises) => {
        // now we need to aggregate all these result sets together
        // into a single array if initiative items
        let allInitiatives = [];
        searchPromises.map((resultSet) => {
          if (resultSet.results.length) {
            // let itemsToDelete = resultSet.results;
            let itemsToDelete = resultSet.results.reduce((acc, entry) => {
              if (entry.title.indexOf('Test Initiative') === 0) {
                acc.push(entry);
              }
              return acc;
            }, []);

            allInitiatives = allInitiatives.concat(itemsToDelete);
          }
        });
        // ok, at this point we should have all the initiatives...
        allInitiatives.map((itm) => {
          console.log(`${itm.id} - ${itm.title}`);
        });
        console.log(`Removing ${allInitiatives.length} Initiatives...`);
        // create the queue...
        let itemQueue = async.queue((task, cb) => {
          // call remove and let it do it's work
          initiativeHelper.remove(task, requestOptions)
          .then((result) => {
            cb();
          }, concurrency);
        });
        // and pump it full of items...
        itemQueue.push(allInitiatives);

        return new Promise((resolve, reject) => {
          itemQueue.drain = function () {
            console.log('Done processing all initiatives');
            resolve({success:true});
          }
        });
      })
      .catch((err) => {
        console.log(`Error Initiatives.process!: ${JSON.stringify(err)}`);
      });
  }
};
